@@ -1,12 +1,12 @@
 import glob
 import os
 
-BNC_CONFIG = os.path.join(os.path.dirname(__file__), 'bnc.cfg')
+BASE_CONFIG = os.path.join(os.path.dirname(__file__), 'base.cfg')
 
 
 class BaseBNC(object):
     def get_config(self):
-        return BNC_CONFIG
+        return BASE_CONFIG
 
     def list_filenames(self, dir_name):
         return sorted(glob.glob(os.path.join(dir_name, '*.xml')))
